const http = require('http')
const express = require('express')
const cors = require('cors')

const { pataka: env } = require('ahau-env')()
const { ApolloServer } = require('apollo-server-express') // TODO replace with express-graphql (copy the ssb-ahau setup)
const { buildFederatedSchema } = require('@apollo/federation')
const Main = require('@ssb-graphql/main')
const Profile = require('@ssb-graphql/profile')
const Invite = require('@ssb-graphql/invite')
const stats = require('@ssb-graphql/stats')
const Pataka = require('@ssb-graphql/pataka')

module.exports = function graphqlServer (ssb) {
  const PORT = env.graphql.port
  const app = express()
  app.options('*', cors())

  const main = Main(ssb, { type: 'pataka' })
  const profile = Profile(ssb)
  const invite = Invite(ssb)
  const pataka = Pataka(ssb, profile.gettersWithCache)

  main.loadContext((err, context) => {
    if (err) throw err

    const apollo = new ApolloServer({
      schema: buildFederatedSchema([
        main,
        profile,
        invite,
        pataka,
        stats
      ]),
      context,
      mockEntireSchema: false
    })
    apollo.applyMiddleware({ app })

    const httpServer = http.createServer(app)
    apollo.installSubscriptionHandlers(httpServer)

    httpServer.listen(PORT, () => {
      console.log(
        `🚀 Apollo ready at http://localhost:${PORT}${apollo.graphqlPath}`
      )
      console.log(
        `🚀 Apollo Subscriptions ready at ws://localhost:${PORT}${apollo.subscriptionsPath}`
      )
    })

    ssb.close.hook((close, args) => {
      httpServer.close()
      close(...args)
    })
  })
}
